/**
 * PETALS - PETALS Services Platform. Copyright (c) 2009 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * -------------------------------------------------------------------------
 * $Id:
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.bc.sftp;

import com.sshtools.j2ssh.SftpClient;
import com.sshtools.j2ssh.SshClient;
import com.sshtools.j2ssh.authentication.AuthenticationProtocolState;
import com.sshtools.j2ssh.authentication.SshAuthenticationClient;
import com.sshtools.j2ssh.configuration.ConfigurationLoader;
import com.sshtools.j2ssh.transport.IgnoreHostKeyVerification;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import org.ow2.petals.bc.sftp.connection.SFTPConnectionInfo;


/**
 * Utility class that holds the SSH connection pool for the component.
 *
 * @author Charles Casadei - eBM WebSourcing
 */
public final class SFTPClientPool {

    /**
     * The actual SFTP connections.
     */
    private static Map<String, SshClient> clients = new HashMap<String, SshClient>();

    /**
     * Returns an SftpClient from an already registered SSH session. If not,
     * this method may return null.
     *
     * Sftp client are instanciated using SshClient, which are stored in a Map,
     * which keys are <code>host+port+login+password</code>. If a client
     * retrieved from the Map is disconnected, another one is created and
     * connected.
     *
     * @param info
     *      connection informations
     * @return
     *      a connected SftpClient
     * @throws SshAuthenticationException
     *      if the authentication failed or if the opened connection does not
     *      give access to a valid SFTP client
     * @throws SftpClientPoolException
     *      if no SFTP client can be retrieved from the SSH connection & the
     *      client pool was not able to remove the SSH client properly
     */
    public static final SftpClient retrieveConnectedSftpClientFromNewSshSession(final SFTPConnectionInfo info) throws SshAuthenticationException, SftpClientPoolException {

        //Builds the key from the given informations
        final String key = info.getKey();

        //Retrieve the SSH client associated to the previous key
        SshClient client = clients.get(key);

        //If the client didn't exist / is not connected anymore / is not authenticated anymore
        if ((client == null) || (!client.isConnected()) || (!client.isAuthenticated())) {
            client = SFTPClientPool.createNewSSHClient(info);
            SFTPClientPool.clients.put(key, client);
        }
        //At this point :
        // - a new SSH connection has been opened
        // - authentication is OK

        //Retrieve the SFTP client
        SftpClient sftpClient = null;
        try {
            try {
                sftpClient = client.openSftpClient();
            } catch (IOException ex) {
                //Throw an exception
                StringBuilder errorSB = new StringBuilder();
                errorSB.append("Can't retrieve SFTP client from the current SSH session ");
                errorSB.append("(this may not be authorized for the current user) : ");
                throw new SshAuthenticationException(errorSB.toString(), ex);
            }
            //At this point :
            // - we may have a reference on a SFTP client
            // - we don't know anything about the connection state

            //If the SFTP client does not exist & no exception has been raised
            if ((sftpClient == null) || (sftpClient.isClosed())) {
                //Throw an exception
                StringBuilder errorSB = new StringBuilder();
                errorSB.append("Retrieved SFTP client is null or closed");
                throw new SshAuthenticationException(errorSB.toString());
            }
            //At this point : 
            // - SFTP client exists & is opened

            //The authentication has succeeded so we can retrieve the SftpClient
            return sftpClient;

        } catch (SshAuthenticationException ex) {
            //Close the SSH connection
            closeSSHSession(info);

            throw ex;
        }
    }

    /**
     * Close a registered SSH session.
     *
     * @param info
     *      connection informations on the sessions this method is about to close
     * @throws SftpClientPoolException
     *      if the given infos doesn't give access to a valid SshClient instance
     */
    public static final void closeSSHSession(final SFTPConnectionInfo info) throws SftpClientPoolException {
        //Builds the key from the given informations
        final String key = info.getKey();

        //Retrieve the SSH client associated to the previous key
        SshClient client = clients.get(key);

        //If the client did not exists in the client pool
        if (client == null) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Unable to close SSH session : the client (");
            errorSB.append(info);
            errorSB.append(") didn't exists in the current pool");
            throw new SftpClientPoolException(errorSB.toString());
        }
        //At this point :
        // - the SSH client exists (if not an exception has been thrown)
        // - we don't know anything about the connection / authentication state

        //If the client is not connected anymore
        if (! client.isConnected()) {
            //Remove the client from the pool
            clients.remove(key);
            return;
        }
        //At this point :
        // - a disconnected client may have been removed from the pool
        // - the SSH client is still connected
        // - the SSH client may not be authenticated anymore

        //If the client is not authenticated anymore
        if (! client.isAuthenticated()) {
            //Close the SSH connection
            client.disconnect();

            //Remove the client from the pool
            clients.remove(key);
        }
        //At this point :
        // - the client is disconnected
        // - the client has been removed from the pool
    }

    /**
     * Return a connected SshClient.
     *
     * Note : If this method fails the SSH connection is automaticaly closed.
     *
     * @param info
     *      required informations to create a new SSH connection
     * @return
     *      a connected SftpClient
     * @throws SshAuthenticationException
     *      raised only if the authentication process failed, the SSH client is
     *      automatically closed. Clients of this method won't have to do it
     *      themselves.
     */
    private static final SshClient createNewSSHClient(final SFTPConnectionInfo info) throws SshAuthenticationException {

        //Configure J2SSH (This will attempt to install the bouncycastle provider under jdk 1.3.1)
        try {
            ConfigurationLoader.initialize(false);
        } catch (com.sshtools.j2ssh.configuration.ConfigurationException ex) { //to avoir name clash
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can't initialize the SSH client : ");
            throw new SshAuthenticationException(errorSB.toString(), ex);
        }
        //At this point :
        // - Bouncy castle may have been installed

        //Make a client connection
        SshClient sshClient = new SshClient();


        try {
            //Try to open a connection to the host
            try {
                sshClient.connect(info.getServer(), info.getPort(), new IgnoreHostKeyVerification());
            } catch (IOException ex) {
                StringBuilder errorSB = new StringBuilder();
                errorSB.append("Can't open a connection to : ");
                errorSB.append(info);
                errorSB.append(" : ");

                throw new SshAuthenticationException(errorSB.toString(), ex);
            }
            //At this point :
            // - SSH client is initalized

            //Tests whether or not the SSH connection is open
            if (!sshClient.isConnected()) {
                StringBuilder errorSB = new StringBuilder();
                errorSB.append("Connection failed.");
                throw new SshAuthenticationException(errorSB.toString());
            }
            //At this point :
            // - SSH client is connected

            //Retrieve the authentication informations
            SshAuthenticationClient sshAuthenticationClient = null;
            try {
                sshAuthenticationClient = info.getSshAuthentication().getAuthenticationClient();
            } catch (SshAuthenticationException ex) {
                throw ex;
            }
            //At this point :
            // - SSH client is connected
            // - We have the authentication provider

            //Try the authentication
            int authenticationResult = 0;
            try {
                authenticationResult = sshClient.authenticate(sshAuthenticationClient);
            } catch (IOException ex) {
                StringBuilder errorSB = new StringBuilder();
                errorSB.append("IO Error during authentication : ");
                throw new SshAuthenticationException(errorSB.toString(), ex);
            }
            //At this point :
            // - SSH client is connected
            // - SSH client may be authenticated...but that's not sure

            //Evalutate the result of the authentication process
            if (authenticationResult != AuthenticationProtocolState.COMPLETE) {
                StringBuilder errorSB = new StringBuilder();
                errorSB.append("Authentication failed.");
                throw new SshAuthenticationException(errorSB.toString());
            }
            //At this point :
            // - SSH client is connected
            // - SSH client is authenticated

        } catch (SshAuthenticationException ex) {
            //If the client is connected > close the connection
            if (sshClient.isConnected()) {
                sshClient.disconnect();
            }

            //Rethrow the original exception
            throw ex;
        }
        //At this point :
        // - if the SSH client has not been connected : an exception has been thrown
        // - if the authentication has failed : the connection has been closed
        // - if none of the above happened : it's OK

        return sshClient;
    }
}
