/**
 * PETALS - PETALS Services Platform. Copyright (c) 2009 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * -------------------------------------------------------------------------
 * $Id:
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.bc.sftp;

import java.util.List;

import javax.jbi.messaging.MessagingException;
import org.apache.commons.lang.CharEncoding;
import javax.jbi.JBIException;
import javax.jbi.messaging.Fault;

import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.util.SourceHelper;

/**
 * This class intends to provide some useful methods like a message exchange 
 * fault setter.
 *
 * @author Charles Casadei - eBM WebSourcing
 */
public class SFTPUtil {

    /**
     * Name of the XML tag used to represent a filename.
     */
    private static final String FILENAME = "filename";

    /**
     * Name of the XML tag used to represent an IO fault.
     */
    private static final String IO_FAULT = "ioFault";

    /**
     * Name of the XML tag used to represent a configuration fault.
     */
    private static final String CONFIGURATION_FAULT = "configurationFault";

    /**
     * Name of the XML tag used to represent a Sftp client pool exception.
     */
    private static final String SFTP_CLIENT_POOL_FAULT = "sftpClientPoolFault";

    /**
     * Name of the XML tag used to represent a SSH Authentication exception.
     */
    private static final String SSH_AUTHENTICATION_FAULT = "sshAuthenticationFault";

    /**
     * Name of the XML tag used to represent a Missing Element fault.
     */
    private static final String MISSING_ELEMENT_FAULT = "missingElementFault";

    /**
     * Generates an XML payload from an operation name an a list of filenames.
     * 
     * @param names filenames which will be contained in the XML payload
     * @param operation the operation which requested to build a filelist
     * @return an XML payload containing a list of filenames
     */
    public static String generateFileNameList(final List<String> names, final String operation) {
        StringBuffer result = new StringBuffer();
        result.append("<tns:" + operation + "Response xmlns:tns='");
        result.append(SFTPConstants.SFTP_SERVICE_NS);
        result.append("'>");
        for (String string : names) {
            result.append("<tns:" + FILENAME + ">");
            result.append(string);
            result.append("</tns:" + FILENAME + ">");
        }
        result.append("</tns:" + operation + "Response>");

        return result.toString();
    }

    /**
     * Sets an IOFault on the given message exchange.
     *
     * @param exchange
     * @param ioMessage
     * @throws javax.jbi.messaging.MessagingException
     * @throws javax.jbi.JBIException
     */
    @SuppressWarnings("deprecation")
    public static void setIOFaultOnExchange(Exchange exchange, String ioMessage) throws MessagingException, JBIException {
        StringBuffer faultMessage = new StringBuffer();
        faultMessage.append("<tns:" + IO_FAULT + " xmlns:tns='");
        faultMessage.append(SFTPConstants.SFTP_SERVICE_NS);
        faultMessage.append("'><tns:message>");
        faultMessage.append(ioMessage);
        faultMessage.append("</tns:message></tns:" + IO_FAULT + ">");
        Fault fault = exchange.createFault();

        try {
            fault.setContent(SourceHelper.createSource(faultMessage.toString(), CharEncoding.UTF_8));
        } catch (PEtALSCDKException e) {
            exchange.setError(e);
            throw new JBIException(e.getMessage());
        }

        exchange.setFault(fault);
    }

    /**
     * Sets a ConfigurationFault on the given message exchange.
     *
     * @param exchange
     * @param configurationError
     * @throws javax.jbi.messaging.MessagingException
     * @throws javax.jbi.JBIException
     */
    @SuppressWarnings("deprecation")
    public static void setConfigurationFaultOnExchange(Exchange exchange, final String configurationError) throws MessagingException, JBIException {
        StringBuffer faultMessage = new StringBuffer();
        faultMessage.append("<tns:" + CONFIGURATION_FAULT + " xmlns:tns='");
        faultMessage.append(SFTPConstants.SFTP_SERVICE_NS);
        faultMessage.append("'><tns:element>");
        faultMessage.append(configurationError);
        faultMessage.append("</tns:element></tns:" + CONFIGURATION_FAULT + ">");
        Fault fault = exchange.createFault();
        try {
            fault.setContent(SourceHelper.createSource(faultMessage.toString(), CharEncoding.UTF_8));
        } catch (PEtALSCDKException e) {
            exchange.setError(e);
            throw new JBIException(e.getMessage());
        }
        
        exchange.setFault(fault);
    }

    /**
     * Sets a client pool fault on the given message exchange.
     *
     * @param exchange
     * @param clientPoolErrorMessage
     * @throws javax.jbi.messaging.MessagingException
     * @throws javax.jbi.JBIException
     */
    @SuppressWarnings("deprecation")
    public static void setClientPoolFaultOnExchange(Exchange exchange, final String clientPoolErrorMessage) throws MessagingException, JBIException {
        StringBuffer faultMessage = new StringBuffer();
        faultMessage.append("<tns:" + SFTP_CLIENT_POOL_FAULT + " xmlns:tns='");
        faultMessage.append(SFTPConstants.SFTP_SERVICE_NS);
        faultMessage.append("'><tns:message>");
        faultMessage.append(clientPoolErrorMessage);
        faultMessage.append("</tns:message></tns:" + SFTP_CLIENT_POOL_FAULT + ">");
        Fault fault = exchange.createFault();
        try {
            fault.setContent(SourceHelper.createSource(faultMessage.toString(), CharEncoding.UTF_8));
        } catch (PEtALSCDKException e) {
            exchange.setError(e);
            throw new JBIException(e.getMessage());
        }

        exchange.setFault(fault);
    }

    /**
     * Sets a SSH authentication fault on the given message exchange.
     *
     * @param exchange
     * @param sshAuthenticationErrorMessage
     * @throws javax.jbi.messaging.MessagingException
     * @throws javax.jbi.JBIException
     */
    @SuppressWarnings("deprecation")
    public static void setSshAuthenticationfaultOnExchange(Exchange exchange, final String sshAuthenticationErrorMessage) throws MessagingException, JBIException {
        StringBuffer faultMessage = new StringBuffer();
        faultMessage.append("<tns:" + SSH_AUTHENTICATION_FAULT + " xmlns:tns='");
        faultMessage.append(SFTPConstants.SFTP_SERVICE_NS);
        faultMessage.append("'><tns:message>");
        faultMessage.append(sshAuthenticationErrorMessage);
        faultMessage.append("</tns:message></tns:" + SSH_AUTHENTICATION_FAULT + ">");
        Fault fault = exchange.createFault();
        try {
            fault.setContent(SourceHelper.createSource(faultMessage.toString(), CharEncoding.UTF_8));
        } catch (PEtALSCDKException e) {
            exchange.setError(e);
            throw new JBIException(e.getMessage());
        }

        exchange.setFault(fault);
    }

    /**
     * Sets a missing element fault on the given message exchange.
     *
     * @param exchange
     * @param missingElement
     * @throws javax.jbi.JBIException
     */
    @SuppressWarnings("deprecation")
    public static void setMissingElementFaultOnExchange(Exchange exchange, final String missingElement)
            throws JBIException {
        StringBuffer faultMessage = new StringBuffer();
        faultMessage.append("<tns:" + MISSING_ELEMENT_FAULT + " xmlns:tns='");
        faultMessage.append(SFTPConstants.SFTP_SERVICE_NS);
        faultMessage.append("'><tns:element>");
        faultMessage.append(missingElement);
        faultMessage.append("</tns:element></tns:" + MISSING_ELEMENT_FAULT + ">");
        Fault fault = exchange.createFault();
        try {
            fault.setContent(SourceHelper.createSource(faultMessage.toString(), CharEncoding.UTF_8));
        } catch (PEtALSCDKException e) {
            exchange.setError(e);
            throw new JBIException(e.getMessage());
        }
        exchange.setFault(fault);
    }

    /**
     * Checks if the MEP of the given message exchange is valid against the
     * given configuration.
     *
     * @param exchange a message exchange
     * @param isInOnly if the InOnly MEP is supported
     * @param isRobustInOnly if the RobustInOnly MEP is supported
     * @param isInOptionalOut if the InOptionalOut MEP is supported
     * @param isInOut if the InOut MEP is supported
     * @return true if the MEP is valid against the given configuration
     */
    public static final boolean checkMEP(final Exchange exchange, 
            final boolean isInOnly,
            final boolean isRobustInOnly,
            final boolean isInOptionalOut,
            final boolean isInOut) {

        //If the MEP InOnly is accepted
        if (isInOnly && exchange.isInOnlyPattern()) {
            return true;
        }

        //If tge MEP RobustInOnly is accepted
        else if (isRobustInOnly && exchange.isRobustInOnlyPattern()) {
            return true;
        }

        //If the MEP InOptionalOut is accepted
        else if (isInOptionalOut && exchange.isInOptionalOutPattern()) {
            return true;
        }

        //If the MEP InOut is accepted
        else if (isInOnly && exchange.isInOutPattern()) {
            return true;
        }

        return false;
    }
}
