/**
 * PETALS - PETALS Services Platform. Copyright (c) 2009 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id:
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.bc.sftp.listeners;

import javax.jbi.JBIException;
import javax.jbi.messaging.MessagingException;
import javax.xml.namespace.QName;

import org.ow2.petals.bc.sftp.ConfigurationException;
import org.ow2.petals.bc.sftp.SFTPConstants;
import org.ow2.petals.bc.sftp.connection.SFTPConnectionInfo;
import org.ow2.petals.bc.sftp.connection.SFTPConnectionInfoBuilder;
import org.ow2.petals.bc.sftp.service.SFTPService;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.listener.AbstractJBIListener;
import org.ow2.petals.component.framework.util.StringHelper;
import org.ow2.petals.component.framework.util.XMLUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
 * The SFTP JBI listener. Writes a file to SFTP server on incoming JBI message.
 * 
 * @author Charles Casadei - eBM WebSourcing
 */
public class SFTPBCJBIListener extends AbstractJBIListener {

    /**
     * A reference on the effective SFTP service.
     */
    SFTPService service = null;

    /**
     * Initialize the JBI Listener.
     */
    @Override
    protected void init() {
        service = new SFTPService(getLogger());
    }

    /**
     * Process the exchange depending on the operation : <li><b>dir</b> return
     * the filenames list or the remote directory</li> <li><b>get</b> return the
     * remote file with the name specified in the exchange (as attachment or
     * source, depending on the SU property)</li> <li><b>mget</b> return all the
     * remote files</li> <li><b>put</b> put the message</li> <li><b>mput</b> put
     * attachments</li>
     * 
     * @param exchange
     *            message received by the component
     */
    @Override
    protected void onJBIMessage(Exchange exchange) {
        // At this point :

        // Ignores the ACK messages
        if (!(exchange.isActiveStatus() && exchange.getFault() == null)) {
            this.getLogger().finest("Receive an ack or a Fault, skip it");
            return;
        }
        // At this point :
        // - the message exchange is not an ACK

        // Process the exchange
        try {
            // Retrieve the operation
            QName operation = exchange.getOperation();
            String operationExchange = null;

            // If the operation is not null
            if (operation != null) {
                operationExchange = operation.getLocalPart();
            }

            // If no operation is set on the message exchnge
            if (StringHelper.isNullOrEmpty(operationExchange)) {
                throw new JBIException("No operation found on the message exchange");
            }

            Document inDocument = exchange.getInMessageContentAsDocument();

            // Retrieve the SFTP configuration informations
            SFTPConnectionInfo connectionInfo = null;
            try {
                connectionInfo = createSFTPConnectionInfo(inDocument);
            } catch (ConfigurationException ex) {
                throw new JBIException(ex.getMessage());
            }

            // Process the incoming operation using its name

            // If operation == DIR
            if (SFTPConstants.DIR.equalsIgnoreCase(operationExchange)) {
                // List the contents of the current remote working directory.
                service.processDIR(exchange, inDocument, connectionInfo);
            } // If operation == PUT
            else if (SFTPConstants.PUT.equalsIgnoreCase(operationExchange)) {
                // Copy incoming XML message on the SFTP server.
                service.processPUT(exchange, inDocument, connectionInfo);
            } else if (SFTPConstants.MPUT.equalsIgnoreCase(operationExchange)) {
                // Copy incoming message attachments on the FTP server.
                service.processMPUT(exchange, inDocument, connectionInfo);
            } else if (SFTPConstants.GET.equalsIgnoreCase(operationExchange)) {
                // Get one specified file from the FTP server and return it as
                // an XML message.
                service.processGET(exchange, inDocument, connectionInfo);
            } else if (SFTPConstants.MGET.equalsIgnoreCase(operationExchange)) {
                // Get files from the FTP server and return them as attachments.
                service.processMGET(exchange, inDocument, connectionInfo);
            } else {
                StringBuilder errorSB = new StringBuilder();
                errorSB.append("Unknown operation : ");
                errorSB.append(operationExchange);
                throw new JBIException(errorSB.toString());
            }

        } catch (JBIException ex) {
            this.getLogger().severe(ex.getMessage());
            exchange.setError(ex);
        }
    }

    /**
     * Search SFTP Connection informations. - first in the message, - then, if
     * not found, in the current ServiceUnit parameters properties
     * 
     * @param exchange
     *            the IN message exchange
     * @return a new SFTPConnectionInfo instance
     * @throws MessagingException
     *             if the MessageExchange is malformed
     * @throws ConfigurationException
     *             if the given configuration is not valid
     */
    private SFTPConnectionInfo createSFTPConnectionInfo(final Document document)
            throws MessagingException, ConfigurationException {

        getLogger().finest("Creating a new  SFTPConnection Information");

        // Retrieve the message content

        // Search a "connection" elements in the children of the document
        Node connection = XMLUtil
                .findChild(document.getFirstChild(), SFTPConstants.SFTP_CONNECTION, provides
                        .getInterfaceName().getNamespaceURI(), false);

        // Creates a new SFTPConnectionInfo object
        SFTPConnectionInfo result = null;
        // If the current exchange has connection informations
        if (connection != null) {
            getLogger().finest(
                    "Creating a new  SFTPConnectionInfo instance from the IN(coming) XML document");
            result = SFTPConnectionInfoBuilder.buildSFTPConnectionInfo(connection);
        }
        // Use the ones provided in the service unit
        else {
            getLogger()
                    .finest(
                            "Creating a new  SFTPConnectionInfo instance from the Service Unit configuration");
            result = SFTPConnectionInfoBuilder.buildSFTPConnectionInfo(getExtensions());
        }

        return result;
    }
}
