/**
 * PETALS - PETALS Services Platform. Copyright (c) 2009 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id:
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.bc.sftp.service;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.logging.Logger;

import javax.activation.DataHandler;
import javax.jbi.JBIException;
import javax.xml.transform.Source;
import javax.xml.transform.TransformerException;
import javax.xml.transform.stream.StreamSource;

import org.ow2.petals.bc.sftp.SFTPConstants;
import org.ow2.petals.bc.sftp.SFTPUtil;
import org.ow2.petals.bc.sftp.SftpClientPoolException;
import org.ow2.petals.bc.sftp.SshAuthenticationException;
import org.ow2.petals.bc.sftp.connection.SFTPConnection;
import org.ow2.petals.bc.sftp.connection.SFTPConnectionInfo;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.util.StringHelper;
import org.ow2.petals.component.framework.util.XMLUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

// TODO :
// - use check in the MEP of the exchange is valid or not
// - replace equalsIgnoreCase by equals

/**
 * The SFTP JBI listener. Writes a file to FTP server on incoming JBI message.
 * 
 * @author Charles Casadei - eBM WebSourcing
 */
public class SFTPService {

    /**
     * The logger used to log informations relative to this class.
     */
    private Logger logger;

    /**
     * Default constructor.
     * 
     * @param logger
     *            logger used to log informations relative to this class.
     */
    public SFTPService(Logger logger) {
        super();
        this.logger = logger;
    }

    /**
     * Lists files of the remote working directory. <br />
     * Supported patterns are : <li>InOut <li>inOptionalOut <br />
     * No attachments are expected <br />
     * No fault / error is set on the message exchange by this method. This
     * method wraps the errors into a JBIException which must be handled at
     * higher level.
     * 
     * @param exchange
     *            the incoming message exchange
     * @param connectionInfo
     *            information on the SFTP server
     * @throws JBIException
     *             if any problem is spotted during execution
     */
    @SuppressWarnings("deprecation")
    public void processDIR(Exchange exchange, Document inDocument, final SFTPConnectionInfo connectionInfo)
            throws JBIException {

        logger.finest("Process DIR operation");

        // At this point :

        // Checks the MEP of the incoming message
        if (!(exchange.isInOutPattern() || exchange.isInOptionalOutPattern())) {
            StringBuilder errorSB = new StringBuilder();
            errorSB
                    .append("Exchange pattern for the DIR operation must be IN-OUT or IN-OPTIONAL-OUT.");
            throw new JBIException(errorSB.toString());
        }
        // At this point :
        // - the message is not null
        // - the message has a valid MEP

        // Check if the message has attachements
        if (!exchange.getInMessageAttachments().isEmpty()) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("No attachement expected.");
            throw new JBIException(errorSB.toString());
        }
        // At this point :
        // - the message is not null
        // - the message has a valid MEP
        // - the message has no attachements

        // Checking the message content
        Element root = inDocument.getDocumentElement();
        // If the first element of the message is not the right one
        if (root == null || !SFTPConstants.DIR.equalsIgnoreCase(root.getLocalName())) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Invalid message content for the DIR operation.");
            throw new JBIException(errorSB.toString());
        }
        // At this point :
        // - the message is not null
        // - the message has a valid MEP
        // - the message has no attachements
        // - the message seems to have a valid root node

        // Performs the DIR operation
        List<String> fileNames = null;
        try {
            // Creates a new connection on the SFTP server
            SFTPConnection sftpConnection = new SFTPConnection(connectionInfo, logger);

            // Retrieve the list of files
            fileNames = sftpConnection.list();

            // generate filename list
            String result = SFTPUtil.generateFileNameList(fileNames, "dir");

            // Sets the exchange OUT content
            exchange.setOutMessageContent(result);
        } catch (SshAuthenticationException ex) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can not retrieve file list from the SFTP server [");
            errorSB.append(connectionInfo);
            errorSB.append("] : SshAuthenticationException : ");
            errorSB.append(ex.getMessage());
            throw new JBIException(errorSB.toString());
        } catch (SftpClientPoolException ex) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can not retrieve file list from the SFTP server [");
            errorSB.append(connectionInfo);
            errorSB.append("] : SftpClientPoolException : ");
            errorSB.append(ex.getMessage());
            throw new JBIException(errorSB.toString());
        } catch (IOException ex) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can not retrieve file list from the SFTP server [");
            errorSB.append(connectionInfo);
            errorSB.append("] : IOException : ");
            errorSB.append(ex.getMessage());
            throw new JBIException(errorSB.toString());
        }
        // At this point :
        // - the DIR operation has been processed or a JBIException has been
        // thrown

        logger.finest("DIR operation ended");
    }

    /**
     * Puts the content on the incoming XML message into the specified file, on
     * the SFTP server. <br />
     * Supported patterns are : <li>InOnly <li>inOptionalOut <li>RobustInOnly <br />
     * No attachments are expected <br />
     * No fault / error is set on the message exchange by this method. This
     * method wraps the errors into a JBIException which must be handled at
     * higher level.
     * 
     * @param exchange
     *            the incoming message exchange
     * @param connectionInfo
     *            information on the SFTP server
     * @throws JBIException
     *             if any problem is spotted during execution
     */
    public void processPUT(Exchange exchange, Document inDocument, final SFTPConnectionInfo connectionInfo)
            throws JBIException {
        logger.finest("Process PUT operation");

        // At this point :

        // Checks the MEP of the incoming message
        if (exchange.isInOutPattern()) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Exchange pattern for the PUT operation must be IN-ONLY, ");
            errorSB.append("IN-OPTIONAL-OUT or ROBUST-IN-ONLY.");
            throw new JBIException(errorSB.toString());
        }
        // At this point :
        // - the message is not null
        // - the message has a valid MEP

        // Check if the message has attachements
        if (!exchange.getInMessageAttachments().isEmpty()) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("No attachement expected.");
            throw new JBIException(errorSB.toString());
        }
        // At this point :
        // - the message is not null
        // - the message has a valid MEP
        // - the message has no attachements

        // Checking the message content
        Node root = inDocument.getFirstChild();
        // If the first element of the message is not the right one
        if (root == null || !SFTPConstants.PUT.equalsIgnoreCase(root.getLocalName())) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Invalid message content for the PUT operation.");
            throw new JBIException(errorSB.toString());
        }
        // At this point :
        // - the message is not null
        // - the message has a valid MEP
        // - the message has no attachements
        // - the message seems to have a valid root node

        // Retrieve the filenames from the message

        // Used to store the filenames
        List<String> filenames = new ArrayList<String>();
        // Retrieve all the node under the root node
        NodeList rootChildNodes = root.getChildNodes();
        // Iterate through the childs of the root node
        for (int i = 0; i < rootChildNodes.getLength(); i++) {
            Node node = rootChildNodes.item(i);
            // If the current node is not null and seems to be a filename
            if ((node != null)
                    && (SFTPConstants.SFTP_FILENAME.equalsIgnoreCase(node.getLocalName()))) {
                // Adds it to the filename list
                filenames.add(node.getTextContent());
            }
        }
        // At this point :
        // - the message is not null
        // - the message has a valid MEP
        // - the message has no attachements
        // - the message seems to have a valid root node
        // - the message has ZERO or MORE filenames node

        // Check if there is at least ONE filename in the list
        if (filenames.size() == 0) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can't retrieve the filename from the XML content : ");
            errorSB.append("No filename node found.");
            throw new JBIException(errorSB.toString());
        }

        // Checks if there is MORE THAN ONE filenames
        if (filenames.size() > 1) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can't retrieve the filename from the XML content : ");
            errorSB.append("Too many filenames nodes found.");
            throw new JBIException(errorSB.toString());
        }
        // At this point :
        // - the message is not null
        // - the message has a valid MEP
        // - the message has no attachements
        // - the message seems to have a valid root node
        // - the message has ONE AND ONLY ONE filename node

        // Checks the filename validity
        String fileName = filenames.get(0);
        // If the filename is null of ""
        if (StringHelper.isNullOrEmpty(fileName)) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can't retrieve the filename from the XML content : ");
            errorSB.append("missing filename.");
            throw new JBIException(errorSB.toString());
        }
        // At this point :
        // - the message is not null
        // - the message has a valid MEP
        // - the message has no attachements
        // - the message seems to have a valid root node
        // - the message has ONE AND ONLY ONE filename node
        // - the ONE AND ONLY ONE filename seems to be OK

        // Check if there is a content node
        Node contentNode = XMLUtil.findChild(root, SFTPConstants.BODY, null, false);
        // If the content node can not be found
        if (contentNode == null || !SFTPConstants.BODY.equalsIgnoreCase(contentNode.getLocalName())) {
            logger.warning("Can't retrieve message body.");
        }
        // At this point :
        // - the message is not null
        // - the message has a valid MEP
        // - the message has no attachements
        // - the message seems to have a valid root node
        // - the filename node seem to have a valid content
        // - the content node seem to have a valid content

        // Performs the PUT operation
        try {
            // Gets the content of the XML message
            String content = XMLUtil.createStringFromDOMNodeList(contentNode.getChildNodes());

            // Creates a connection to the SSH server using the given
            // configuration
            SFTPConnection sftpConnection = new SFTPConnection(connectionInfo, logger);

            // Put the content to the SSH server
            sftpConnection.putString(fileName, content);
        } catch (SshAuthenticationException ex) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can not put XML content on SFTP server [");
            errorSB.append(connectionInfo);
            errorSB.append("] : SshAuthenticationException : ");
            errorSB.append(ex.getMessage());
            throw new JBIException(errorSB.toString());
        } catch (SftpClientPoolException ex) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can not put XML content on SFTP server [");
            errorSB.append(connectionInfo);
            errorSB.append("] : SftpClientPoolException : ");
            errorSB.append(ex.getMessage());
            throw new JBIException(errorSB.toString());
        } catch (IOException ex) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can not put XML content on SFTP server [");
            errorSB.append(connectionInfo);
            errorSB.append("] : IOException : ");
            errorSB.append(ex.getMessage());
            throw new JBIException(errorSB.toString());
        } catch (TransformerException ex) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can not put XML content on SFTP server [");
            errorSB.append(connectionInfo);
            errorSB.append("] : TransformerException : ");
            errorSB.append(ex.getMessage());
            throw new JBIException(errorSB.toString());
        }
        // At this point :
        // - the PUT operation has been processed or a JBIException has been
        // thrown

        logger.finest("PUT operation ended");
    }

    @SuppressWarnings("unchecked")
    public void processMPUT(Exchange exchange, Document inDocument, final SFTPConnectionInfo connectionInfo)
            throws JBIException {
        logger.finest("Process MPUT operation");

        // Checks the MEP of the incoming message
        if (exchange.isInOutPattern()) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Exchange pattern for the MPUT operation must be IN-ONLY, ");
            errorSB.append("IN-OPTIONAL-OUT or ROBUST-IN-ONLY.");
            throw new JBIException(errorSB.toString());
        }
        // At this point :
        // - the message is not null
        // - the message has a valid MEP

        // Check if the message has attachements
        if (exchange.getInMessageAttachments().isEmpty()) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("One or more attachments expected.");
            throw new JBIException(errorSB.toString());
        }
        // At this point :
        // - the message is not null
        // - the message has a valid MEP
        // - the message has attachements

        // Checking the message content
        Element root = inDocument.getDocumentElement();
        // If the first element of the message is not the right one
        if (root == null || !SFTPConstants.MPUT.equalsIgnoreCase(root.getLocalName())) {
            throw new JBIException("Invalid message content for the MPUT operation.");
        }
        // At this point :
        // - the message is not null
        // - the message has a valid MEP
        // - the message has attachments
        // - the message seems to have a valid root node

        // For the next iteration on this component
        // //Retrieve the filenames from the message
        //
        // //Used to store the filenames
        // List<String> filenames = new ArrayList<String>();
        // //Retrieve all the node under the root node
        // NodeList rootChildNodes = root.getChildNodes();
        // //Iterate through the children of the root node
        // for (int i = 0 ; i < rootChildNodes.getLength() ; i++) {
        // Node node = rootChildNodes.item(i);
        // //If the current node is not null and seems to be a filename
        // if ((node != null) &&
        // (SFTPConstants.SFTP_FILENAME.equalsIgnoreCase(node.getLocalName())))
        // {
        // //Adds it to the filename list
        // filenames.add(node.getTextContent());
        // }
        // }
        // //At this point :
        // // - the message is not null
        // // - the message has a valid MEP
        // // - the message has attachments
        // // - the message seems to have a valid root node
        // // - the message has ZERO or MORE filenames node
        //
        // //Check if there is at least ONE filename in the list
        // if (filenames.size() == 0) {
        // StringBuilder errorSB = new StringBuilder();
        // errorSB.append("Can't retrieve the filename from the XML content : ");
        // errorSB.append("No filename node found.");
        // throw new JBIException(errorSB.toString());
        // }
        // //At this point :
        // // - the message is not null
        // // - the message has a valid MEP
        // // - the message has attachments
        // // - the message seems to have a valid root node
        // // - the message has ONE or MORE filenames node
        //
        // //Checks the filenames validity
        // for (String filename : filenames) {
        // if (StringHelper.isNullOrEmpty(filename)) {
        // StringBuilder errorSB = new StringBuilder();
        // errorSB.append("Can't retrieve the filename from the XML content : ");
        // errorSB.append("missing filename.");
        // throw new JBIException(errorSB.toString());
        // }
        // }
        // //At this point :
        // // - the message is not null
        // // - the message has a valid MEP
        // // - the message has attachements
        // // - the message seems to have a valid root node
        // // - the message has ONE or MORE filenames node
        // // - the filenames founds in the message are not null or ""

        // Perform the MPUT operation
        try {
            // Creates a new connection on the SFTP server
            SFTPConnection sftpConnection = new SFTPConnection(connectionInfo, logger);

            // Holds the attachments names
            Set<String> attachmentsNames = (Set<String>) exchange.getInMessage()
                    .getAttachmentNames();

            // Holds the rebuilt a hashmap containing files and their names
            HashMap<String, DataHandler> attachments = new HashMap<String, DataHandler>();

            // Rebuilds a hashmap containing files and their names
            for (String fileName : attachmentsNames) {
                attachments.put(fileName, exchange.getInMessage().getAttachment(fileName));
            }

            // TODO : wildcard management
            // TODO : what if we don't want to put all the attached files to the
            // SFTP server

            // Sends the attachments to the SFTP server
            sftpConnection.mput(attachments);

        } catch (SshAuthenticationException ex) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can not put files [");
            errorSB.append(connectionInfo);
            errorSB.append("] : SshAuthenticationException : ");
            errorSB.append(ex.getMessage());
            throw new JBIException(errorSB.toString());
        } catch (SftpClientPoolException ex) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can not put file [");
            errorSB.append(connectionInfo);
            errorSB.append("] : SftpClientPoolException : ");
            errorSB.append(ex.getMessage());
            throw new JBIException(errorSB.toString());
        } catch (IOException ex) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can not put file [");
            errorSB.append(connectionInfo);
            errorSB.append("] : IOException : ");
            errorSB.append(ex.getMessage());
            throw new JBIException(errorSB.toString());
        }
        // At this point :
        // - the MPUT operation has been processed or a JBIException has been
        // thrown

        logger.finest("MPUT operation ended");
    }

    /**
     * Gets the specified file from the SFTP server. <br />
     * Supported patterns are : <li>InOut <li>inOptionalOut <br />
     * No attachments are expected <br />
     * No fault / error is set on the message exchange by this method. This
     * method wraps the errors into a JBIException which must be handled at
     * higher level.
     * 
     * @param exchange
     *            the incoming message exchange
     * @param connectionInfo
     *            information on the SFTP server
     * @throws JBIException
     *             if any problem is spotted during execution
     */
    public void processGET(Exchange exchange, Document inDocument, final SFTPConnectionInfo connectionInfo)
            throws JBIException {
        logger.finest("Process GET operation");

        // At this point :

        // Checks the MEP of the incoming message
        if (!(exchange.isInOutPattern() || exchange.isInOptionalOutPattern())) {
            StringBuilder errorSB = new StringBuilder();
            errorSB
                    .append("Exchange pattern for the GET operation must be IN-OUT or IN-OPTIONAL-OUT.");
            throw new JBIException(errorSB.toString());
        }
        // At this point :
        // - the message is not null
        // - the message has a valid MEP

        // Check if the message has attachements
        if (!exchange.getInMessageAttachments().isEmpty()) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("No attachement expected.");
            throw new JBIException(errorSB.toString());
        }
        // At this point :
        // - the message is not null
        // - the message has a valid MEP
        // - the message has no attachements

        // Checking the message content
        Node root = inDocument.getFirstChild();
        // If the first element of the message is not the right one
        if (root == null || !SFTPConstants.GET.equalsIgnoreCase(root.getLocalName())) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Invalid message content for the GET operation.");
            throw new JBIException(errorSB.toString());
        }
        // At this point :
        // - the message is not null
        // - the message has a valid MEP
        // - the message has no attachements
        // - the message seems to have a valid root node

        // Retrieve the filenames from the message

        // Used to store the filenames
        List<String> filenames = new ArrayList<String>();
        // Retrieve all the node under the root node
        NodeList rootChildNodes = root.getChildNodes();
        // Iterate through the childs of the root node
        for (int i = 0; i < rootChildNodes.getLength(); i++) {
            Node node = rootChildNodes.item(i);
            // If the current node is not null and seems to be a filename
            if ((node != null)
                    && (SFTPConstants.SFTP_FILENAME.equalsIgnoreCase(node.getLocalName()))) {
                // Adds it to the filename list
                filenames.add(node.getTextContent());
            }
        }
        // At this point :
        // - the message is not null
        // - the message has a valid MEP
        // - the message has no attachements
        // - the message seems to have a valid root node
        // - the message has ZERO or MORE filenames node

        // Check if there is at least ONE filename in the list
        if (filenames.size() == 0) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can't retrieve the filename from the XML content : ");
            errorSB.append("No filename node found.");
            throw new JBIException(errorSB.toString());
        }

        // Checks if there is MORE THAN ONE filenames
        if (filenames.size() > 1) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can't retrieve the filename from the XML content : ");
            errorSB.append("Too many filenames nodes found.");
            throw new JBIException(errorSB.toString());
        }
        // At this point :
        // - the message is not null
        // - the message has a valid MEP
        // - the message has no attachements
        // - the message seems to have a valid root node
        // - the message has ONE AND ONLY ONE filename node

        // Checks the filename validity
        String fileName = filenames.get(0);
        // If the filename is null of ""
        if (StringHelper.isNullOrEmpty(fileName)) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can't retrieve the filename from the XML content : ");
            errorSB.append("missing filename.");
            throw new JBIException(errorSB.toString());
        }
        // At this point :
        // - the message is not null
        // - the message has a valid MEP
        // - the message has no attachements
        // - the message seems to have a valid root node
        // - the message has ONE AND ONLY ONE filename node
        // - the ONE AND ONLY ONE filename seems to be OK

        // Perform the GET operation
        try {
            // Creates a new connection on the SFTP server
            SFTPConnection sftpConnection = new SFTPConnection(connectionInfo, logger);

            // Gets the file from the SFTP server
            DataHandler dh = sftpConnection.get(fileName);
            Source stream = new StreamSource(dh.getInputStream());

            // Put the file in the message
            exchange.setOutMessageContent(stream);

        } catch (SshAuthenticationException ex) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can not get file : ");
            errorSB.append(fileName + " [");
            errorSB.append(connectionInfo);
            errorSB.append("] : SshAuthenticationException : ");
            errorSB.append(ex.getMessage());
            throw new JBIException(errorSB.toString());
        } catch (SftpClientPoolException ex) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can not get file : ");
            errorSB.append(fileName + " [");
            errorSB.append(connectionInfo);
            errorSB.append("] : SftpClientPoolException : ");
            errorSB.append(ex.getMessage());
            throw new JBIException(errorSB.toString());
        } catch (IOException ex) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can not get file : ");
            errorSB.append(fileName + " [");
            errorSB.append(connectionInfo);
            errorSB.append("] : IOException : ");
            errorSB.append(ex.getMessage());
            throw new JBIException(errorSB.toString());
        }
        // At this point :
        // - the GET operation has been processed or a JBIException has been
        // thrown

        logger.finest("GET operation ended");
    }

    /**
     * Gets the specified files from the SFTP server. <br />
     * Supported patterns are : <li>InOut <li>inOptionalOut <br />
     * No attachments are expected <br />
     * No fault / error is set on the message exchange by this method. This
     * method wraps the errors into a JBIException which must be handled at
     * higher level.
     * 
     * @param exchange
     *            the incoming message exchange
     * @param connectionInfo
     *            information on the SFTP server
     * @throws JBIException
     *             if any problem is spotted during execution
     */
    @SuppressWarnings("deprecation")
    public void processMGET(Exchange exchange, Document inDocument, final SFTPConnectionInfo connectionInfo)
            throws JBIException {
        logger.finest("Process MGET operation");

        // At this point :

        // Checks the MEP of the incoming message
        if (!(exchange.isInOutPattern() || exchange.isInOptionalOutPattern())) {
            StringBuilder errorSB = new StringBuilder();
            errorSB
                    .append("Exchange pattern for the MGET operation must be IN-OUT or IN-OPTIONAL-OUT.");
            throw new JBIException(errorSB.toString());
        }
        // At this point :
        // - the message is not null
        // - the message has a valid MEP

        // Check if the message has attachements
        if (!exchange.getInMessageAttachments().isEmpty()) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("No attachement expected.");
            throw new JBIException(errorSB.toString());
        }
        // At this point :
        // - the message is not null
        // - the message has a valid MEP
        // - the message has no attachements

        // Checking the message content
        Node root = inDocument.getFirstChild();
        // If the first element of the message is not the right one
        if (root == null || !SFTPConstants.MGET.equalsIgnoreCase(root.getLocalName())) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Invalid message content for the GET operation.");
            throw new JBIException(errorSB.toString());
        }
        // At this point :
        // - the message is not null
        // - the message has a valid MEP
        // - the message has no attachements
        // - the message seems to have a valid root node

        // Retrieve the filenames from the message

        // Used to store the filenames
        List<String> filenames = new ArrayList<String>();
        // Retrieve all the node under the root node
        NodeList rootChildNodes = root.getChildNodes();
        // Iterate through the childs of the root node
        for (int i = 0; i < rootChildNodes.getLength(); i++) {
            Node node = rootChildNodes.item(i);
            // If the current node is not null and seems to be a filename
            if ((node != null)
                    && (SFTPConstants.SFTP_FILENAME.equalsIgnoreCase(node.getLocalName()))) {
                // Adds it to the filename list
                filenames.add(node.getTextContent());
            }
        }
        // At this point :
        // - the message is not null
        // - the message has a valid MEP
        // - the message has no attachements
        // - the message seems to have a valid root node
        // - the message has ZERO or MORE filenames node

        // Check if there is at least ONE filename in the list
        if (filenames.size() == 0) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can't retrieve the filename from the XML content : ");
            errorSB.append("No filename node found.");
            throw new JBIException(errorSB.toString());
        }
        // At this point :
        // - the message is not null
        // - the message has a valid MEP
        // - the message has no attachements
        // - the message seems to have a valid root node
        // - the message has ONE or MORE filenames node

        // Checks the filenames validity
        for (String filename : filenames) {
            if (StringHelper.isNullOrEmpty(filename)) {
                StringBuilder errorSB = new StringBuilder();
                errorSB.append("Can't retrieve the filename from the XML content : ");
                errorSB.append("missing filename.");
                throw new JBIException(errorSB.toString());
            }
        }
        // At this point :
        // - the message is not null
        // - the message has a valid MEP
        // - the message has no attachements
        // - the message seems to have a valid root node
        // - the message has ONE or MORE filenames node
        // - the filenames founds in the message are not null or ""

        // Perform the MGET operation
        try {
            // Creates a new connection on the SFTP server
            SFTPConnection sftpConnection = new SFTPConnection(connectionInfo, logger);

            // Perform the MGET operation
            Map<String, DataHandler> files = sftpConnection.mGet(filenames);

            // Add attachments on the OUT message
            for (String filename : files.keySet()) {
                exchange.getOutMessage().addAttachment(filename, files.get(filename));
            }

            // Builds the list of the retrieved files
            List<String> retrievedFilenames = new LinkedList<String>(files.keySet());

            // Builds the response content
            String response = SFTPUtil.generateFileNameList(retrievedFilenames, "mget");

            // Sends the response
            exchange.setOutMessageContent(response);
        } catch (SshAuthenticationException ex) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can not get files [");
            errorSB.append(connectionInfo);
            errorSB.append("] : SshAuthenticationException : ");
            errorSB.append(ex.getMessage());
            throw new JBIException(errorSB.toString());
        } catch (SftpClientPoolException ex) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can not get file [");
            errorSB.append(connectionInfo);
            errorSB.append("] : SftpClientPoolException : ");
            errorSB.append(ex.getMessage());
            throw new JBIException(errorSB.toString());
        } catch (IOException ex) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can not get file [");
            errorSB.append(connectionInfo);
            errorSB.append("] : IOException : ");
            errorSB.append(ex.getMessage());
            throw new JBIException(errorSB.toString());
        }
        // At this point :
        // - the MGET operation has been processed or a JBIException has been
        // thrown

        logger.finest("MGET operation ended");
    }
}
